package sun.java2d.loops;

import java.lang.ref.WeakReference;
import sun.java2d.pipe.Region;
import java.awt.Composite;
import sun.java2d.SurfaceData;

public class MaskBlit extends GraphicsPrimitive
{
    public static final String methodSignature;
    public static final int primTypeID;
    private static RenderCache blitcache;
    
    public static MaskBlit locate(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return (MaskBlit)GraphicsPrimitiveMgr.locate(MaskBlit.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public static MaskBlit getFromCache(SurfaceType obj, CompositeType obj2, SurfaceType obj3) {
        Object value = MaskBlit.blitcache.get(obj, obj2, obj3);
        if (value != null) {
            return (MaskBlit)value;
        }
        MaskBlit locate = locate(obj, obj2, obj3);
        if (locate == null) {
            System.out.println("mask blit loop not found for:");
            System.out.println("src:  " + obj);
            System.out.println("comp: " + obj2);
            System.out.println("dst:  " + obj3);
            return locate;
        }
        MaskBlit.blitcache.put(obj, obj2, obj3, locate);
        return locate;
    }
    
    protected MaskBlit(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(MaskBlit.methodSignature, MaskBlit.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public MaskBlit(long n, SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(n, MaskBlit.methodSignature, MaskBlit.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public native void MaskBlit(SurfaceData p0, SurfaceData p1, Composite p2, Region p3, int p4, int p5, int p6, int p7, int p8, int p9, byte[] p10, int p11, int p12);
    
    @Override
    public GraphicsPrimitive makePrimitive(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        if (CompositeType.Xor.equals(compositeType)) {
            throw new InternalError("Cannot construct MaskBlit for XOR mode");
        }
        MaskBlit.General general = new General(surfaceType, compositeType, surfaceType2);
        this.setupGeneralBinaryOp(general);
        return general;
    }
    
    @Override
    public GraphicsPrimitive traceWrap() {
        return new MaskBlit.TraceMaskBlit(this);
    }
    
    static {
        methodSignature = "MaskBlit(...)".toString();
        primTypeID = GraphicsPrimitive.makePrimTypeID();
        MaskBlit.blitcache = new RenderCache(20);
        GraphicsPrimitiveMgr.registerGeneral(new MaskBlit(null, null, null));
    }
    
    private static class General extends MaskBlit implements GraphicsPrimitive.GeneralBinaryOp
    {
        Blit convertsrc;
        Blit convertdst;
        MaskBlit performop;
        Blit convertresult;
        WeakReference srcTmp;
        WeakReference dstTmp;
        
        public General(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
            super(surfaceType, compositeType, surfaceType2);
        }
        
        @Override
        public void setPrimitives(Blit convertsrc, Blit convertdst, GraphicsPrimitive graphicsPrimitive, Blit convertresult) {
            this.convertsrc = convertsrc;
            this.convertdst = convertdst;
            this.performop = (MaskBlit)graphicsPrimitive;
            this.convertresult = convertresult;
        }
        
        @Override
        public synchronized void MaskBlit(SurfaceData convert, SurfaceData convert2, Composite composite, Region region, int n, int n2, int n3, int n4, int n5, int n6, byte[] array, int n7, int n8) {
            if (this.convertsrc == null) {
            }
            else {
                SurfaceData surfaceData = null;
                if (this.srcTmp != null) {
                    surfaceData = (SurfaceData)this.srcTmp.get();
                }
                convert = GraphicsPrimitive.convertFrom(this.convertsrc, convert, n, n2, n5, n6, surfaceData);
                n = 0;
                n2 = 0;
                if (convert != surfaceData) {
                    this.srcTmp = new WeakReference(convert);
                }
            }
            if (this.convertdst == null) {
            }
            else {
                SurfaceData surfaceData2 = null;
                if (this.dstTmp != null) {
                    surfaceData2 = (SurfaceData)this.dstTmp.get();
                }
                convert2 = GraphicsPrimitive.convertFrom(this.convertdst, convert2, n3, n4, n5, n6, surfaceData2);
                n3 = 0;
                n4 = 0;
                region = null;
                if (convert2 != surfaceData2) {
                    this.dstTmp = new WeakReference(convert2);
                }
            }
            this.performop.MaskBlit(convert, convert2, composite, region, n, n2, n3, n4, n5, n6, array, n7, n8);
            if (this.convertresult != null) {
                GraphicsPrimitive.convertTo(this.convertresult, convert2, convert2, region, n3, n4, n5, n6);
            }
        }
    }
    
    private static class TraceMaskBlit extends MaskBlit
    {
        MaskBlit target;
        
        public TraceMaskBlit(MaskBlit target) {
            super(target.getNativePrim(), target.getSourceType(), target.getCompositeType(), target.getDestType());
            this.target = target;
        }
        
        @Override
        public GraphicsPrimitive traceWrap() {
            return this;
        }
        
        @Override
        public void MaskBlit(SurfaceData surfaceData, SurfaceData surfaceData2, Composite composite, Region region, int n, int n2, int n3, int n4, int n5, int n6, byte[] array, int n7, int n8) {
            GraphicsPrimitive.tracePrimitive(this.target);
            this.target.MaskBlit(surfaceData, surfaceData2, composite, region, n, n2, n3, n4, n5, n6, array, n7, n8);
        }
    }
}
